// Window1.xaml.cs
using System;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Collections.Generic;
using System.Diagnostics;
using System.Collections;
using System.Data;
using System.Data.OleDb;
using System.ComponentModel;
using System.Xml;

namespace XmlBinding {

  public partial class Window1 : Window {

    // XML document ü Ѵ.
    XmlDocument doc;

    public Window1() {
      InitializeComponent();

      this.birthdayButton.Click += birthdayButton_Click;
      this.backButton.Click += backButton_Click;
      this.forwardButton.Click += forwardButton_Click;
      this.addButton.Click += addButton_Click;
      this.sortButton.Click += sortButton_Click;
      this.filterButton.Click += filterButton_Click;
      this.groupButton.Click += groupButton_Click;

      LoadFamilyXml();
    }

    void LoadFamilyXml() {
      // XmlDocument ü ̿Ͽ XML εѴ.
      doc = new XmlDocument();
      doc.Load("family.xml");

      // ε  ӽ̽ ξ Ѵ.
      XmlNamespaceManager manager = new XmlNamespaceManager(doc.NameTable);
      manager.AddNamespace("sb", "http://sellsbrothers.com");
      Binding.SetXmlNamespaceManager(grid, manager);
	  
	  // XML  ε ϵ .  XML 
	  //   ֱ  XPath   ڽ 带  ĥ  Ƿ
	  // ⿡ ε ص ϴ.
      Binding b = new Binding();
      b.XPath = "/sb:Family/sb:Person";
      b.Source = doc;
      grid.SetBinding(Grid.DataContextProperty, b);
    }

    ICollectionView GetFamilyView() {
      // ⺻  ͷκ  ´.
      return CollectionViewSource.GetDefaultView(grid.DataContext);
    }

    void birthdayButton_Click(object sender, RoutedEventArgs e) {
      ICollectionView view = GetFamilyView();

      //  "person" XmlElement ü̸   ڿ ̿ϴ ε  ´.
      XmlElement person = (XmlElement)view.CurrentItem;
      person.SetAttribute("Age",
        (int.Parse(person.Attributes["Age"].Value) + 1).ToString());
      MessageBox.Show(
        string.Format(
          " ؿ, {0}!,  : {1}!",
          person.Attributes["Name"].Value,
          person.Attributes["Age"].Value),
        "");
    }

    void backButton_Click(object sender, RoutedEventArgs e) {
      ICollectionView view = GetFamilyView();
      view.MoveCurrentToPrevious();
      if( view.IsCurrentBeforeFirst ) {
        view.MoveCurrentToFirst();
      }
    }

    void forwardButton_Click(object sender, RoutedEventArgs e) {
      ICollectionView view = GetFamilyView();
      view.MoveCurrentToNext();
      if( view.IsCurrentAfterLast ) {
        view.MoveCurrentToLast();
      }
    }

    void addButton_Click(object sender, RoutedEventArgs e) {
      //  XmlElement ü .
      XmlElement person = doc.CreateElement("Person", "http://sellsbrothers.com");
      person.SetAttribute("Name", "Chris");
      person.SetAttribute("Age", "37");
      doc.DocumentElement.AppendChild(person);
    }

    void sortButton_Click(object sender, RoutedEventArgs e) {
      ICollectionView view = GetFamilyView();
      if( view.SortDescriptions.Count == 0 ) {
        view.SortDescriptions.Add(
          new SortDescription("@Name", ListSortDirection.Ascending));
        view.SortDescriptions.Add(
          new SortDescription("@Age", ListSortDirection.Descending));
      }
      else {
        view.SortDescriptions.Clear();
      }
    }

    void filterButton_Click(object sender, RoutedEventArgs e) {
      ICollectionView view = GetFamilyView();

      if( view.Filter == null ) {
        view.Filter = delegate(object item) {
          return
            int.Parse(((XmlElement)item).Attributes["Age"].Value) > 25;
        };
      }
      else {
        view.Filter = null;
      }
    }

    void groupButton_Click(object sender, RoutedEventArgs e) {
      ICollectionView view = GetFamilyView();
      if( view.GroupDescriptions.Count == 0 ) {
        // ̺ ׷ȭѴ.
        view.GroupDescriptions.Add(new PropertyGroupDescription("@Age"));
      }
      else {
        view.GroupDescriptions.Clear();
      }
    }

  }

  public class AgeToForegroundConverter : IValueConverter {
    public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture) {
      Debug.Assert(targetType == typeof(Brush));

      int age = int.Parse(value.ToString());
      return (age > 25 ? Brushes.Red : Brushes.Black);
    }

    public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture) {
      throw new NotImplementedException();
    }
  }

}
